#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Poincaré photographic tiling of the poincaré disk  http://poincare.sourceforge.net/
# Notepad2 - free and open source Copyright © 2004-2011 Florian Balmer: http://www.flos-freeware.ch/notepad2.html
# Example: http://gimpchat.com/viewtopic.php?f=22&t=1825&start=0

from gimpfu import *
import shlex
import subprocess
import os, sys

def plugin_main_save_orig(img, layer, outputFolder):
  gimp.pdb.file_jpeg_save(img, layer, outputFolder + "\\" + "image_orig" + ".jpg", "raw_filename", 0.9, 0, 0, 0, "Creating with GIMP", 0, 0, 0, 0, run_mode=RUN_INTERACTIVE)
  command = "\"C:\\poincare\\poincare_mask_edit.bat\""
  args = shlex.split(command)
  child = subprocess.Popen(args, shell=False)
  return
  
register(
        "python_fu_poincare_save_orig",
	    "In general, generating one of these images is a multistep process.\nFirst, you have to identify the “fundamental region” from an image which you want to base the output on.\nThe fundamental region is a triangular-shaped region (actually, it’s a hyperbolic triangle ).\nThis is an iterative process that not particularly intuitive.\nYou have to determine four key points (px, py, rx, and ry) in the parameter file.\nEach time you set those parmeters you will generate an output file which shows the masked input image.\nYou will find that you will have to keep tweaking those parameters, until you get the mask just right.\nI often do a dozen or more iterations before I get this step right. \n\nSTEP 1. This plug-ins:\n\nSave active layer as image_orig.jpeg and open (in notepad) parameters mask for edit",	
        "PoincareSwith 0.1",
        "RobA/Lylejk/MrQ",
        "GPL",
        "2014",
        "<Image>/Filters/Render/ Poincare/1. Save layer as image-orig.jpg and open parameters  mask...",
        "*", 
        [

        (PF_DIRNAME, "outputFolder", "Output directory:", "C:\\poincare\\"),  # << - EDIT path to the directory of the program: poincare.exe
        ],   
        [],
        plugin_main_save_orig,
        )

def plugin_render_mask(img, layer, copy_mask):
  command1 = "\"C:\\poincare\\poincare_mask.bat\""
  args = shlex.split(command1)
  child = subprocess.Popen(args, shell=False)
  if copy_mask:
   command = "\"C:\\poincare\\poincare_mask_txt_copy.bat\""
   args = shlex.split(command)
   child = subprocess.Popen(args, shell=False)   

   return
  
register(
        "python_fu_render_mask",
	    "STEP 2.This plug-ins:\n\nCreate image mask (generate just the fundamental region)   ",	
        "PoincareSwith 0.1",
        "RobA/Lylejk/MrQ",
        "GPL",
        "2014",
        "<Image>/Filters/Render/ Poincare/2. Render and save image-mask.jpg.....",
        "*", 
        [
        (PF_BOOL, "copy_mask", "Save parameters mask as copy ?", False),
        ],   
        [],
        plugin_render_mask,
        )

  
def open_mask (img, layer, file):

  gimp.progress_init("Opening '" + file + "'...")
  fileImage = pdb.file_jpeg_load(file, file, run_mode=RUN_NONINTERACTIVE)
  fileLayer = fileImage.layers[0]
  pdb.gimp_edit_copy(fileLayer)
  pdb.gimp_floating_sel_to_layer(pdb.gimp_edit_paste(layer, False))
  command = "\"C:\\poincare\\poincare_render_edit.bat\""
  args = shlex.split(command)
  child = subprocess.Popen(args, shell=False)  

register(
        "python_fu_poincare_open_mask",
	    "Next, only once you have identified the fundamental region can you render the tiling of the poincare disk.  This step is straightforward, although you may want to do this with a small value for the supersampling rate, say 1, or decrease the size of the rendered image so the program runs quickly.  Then, once you are satisfied, increase the supersampling rate to, say 4, to get a final image, but will take much more CPU time to do.\n\nSTEP 3.This plug-ins:\n\nOpen image mask for layer and open (in notepad) parameters render ...",	
        "PoincareSwith 0.1",
        "RobA/Lylejk/MrQ",
        "GPL",
        "2014",
        "<Image>/Filters/Render/ Poincare/3. Open image mask as layer an edit parameters render...",
        "*", 
        [
        (PF_FILE, "file", "File to open:" ,"C:\\poincare\\image_mask.jpg"),
        ],   
        [],
        open_mask,
        )  

def plugin_poincare_render(img, layer, copy_render):
  command1 = "\"C:\\poincare\\poincare_render.bat\""
  args = shlex.split(command1)
  child = subprocess.Popen(args, shell=False)
  if copy_render:
   command = "\"C:\\poincare\\poincare_render_txt_copy.bat\""
   args = shlex.split(command)
   child = subprocess.Popen(args, shell=False)

   return
  
register(
        "python_fu_poincare_render",
	    "STEP 4.This plug-ins:\n\nRenderin poincare image (generate a tiling of the disk)    ",	
        "PoincareSwith 0.1",
        "RobA/Lylejk/MrQ",
        "GPL",
        "2014",
        "<Image>/Filters/Render/ Poincare/4. Render and save image-poincare.jpg...",
        "*", 
        [
        (PF_BOOL, "copy_render", "Save parameters mask as copy ?", False),
        ],   
        [],
        plugin_poincare_render,
        )
		
def open_image(img, layer, file):

    gimp.progress_init("Opening '" + file + "'...")
    newImage = pdb.file_jpeg_load(file, file, run_mode=RUN_NONINTERACTIVE)
    pdb.gimp_display_new(newImage)
    return
    
register(
    "python_fu_poincare_open_jpg",
    "STEP 5.This plug-ins:\nOpenes poincare image as new image    ",
    "PoincareSwith 0.1.",
    "MrQ",
    "GPL",
    "2014",
    "<Image>/Filters/Render/ Poincare/5. Open image-poincare.jpg as new image...",
    "*",
    [
    (PF_FILE, "file", "File to open", "C:\\poincare\\image_poincare.jpg"),
    ],
    [],
    open_image)
	


main()
